@@ -1,7 +1,10 @@
 use crate::construction::constraints::TransportConstraintModule;
 use crate::construction::heuristics::InsertionContext;
+use crate::models::solution::Activity;
 use crate::solver::mutation::LocalOperator;
 use crate::solver::RefinementContext;
+use crate::utils::compare_floats;
+use std::cmp::Ordering;
 
 /// Reschedules departure time of the routes in the solution.
 pub struct RescheduleDeparture {}
@@ -22,9 +25,18 @@ impl LocalOperator for RescheduleDeparture {
 
         // TODO optionally, optimize only subset of the routes.
 
+        let random = insertion_ctx.environment.random.clone();
+
         let mut insertion_ctx = insertion_ctx.deep_copy();
         insertion_ctx.solution.routes.iter_mut().for_each(|route_ctx| {
-            TransportConstraintModule::optimize_departure_time(route_ctx, transport);
+            let earliest = route_ctx.route.actor.detail.start.as_ref().and_then(|start| start.time.earliest);
+
+            match (route_ctx.route.tour.start(), earliest, random.is_head_not_tails()) {
+                (Some(start), Some(earliest), true) if can_recede_departure(start, earliest) => {
+                    TransportConstraintModule::recede_departure_time(route_ctx, transport)
+                }
+                _ => TransportConstraintModule::advance_departure_time(route_ctx, transport, true),
+            };
         });
 
         // TODO check is_stale flag and return None
@@ -34,3 +46,7 @@ impl LocalOperator for RescheduleDeparture {
         Some(insertion_ctx)
     }
 }
+
+fn can_recede_departure(start: &Activity, earliest: f64) -> bool {
+    compare_floats(start.schedule.departure, earliest) != Ordering::Equal
+}
