@@ -1,5 +1,5 @@
 mod timing {
-    use super::super::try_get_new_departure_time;
+    use super::super::{try_advance_departure_time, try_recede_departure_time};
     use crate::construction::constraints::*;
     use crate::construction::heuristics::*;
     use crate::helpers::construction::constraints::create_constraint_pipeline_with_transport;
@@ -149,7 +149,7 @@ mod timing {
         create_constraint_pipeline_with_transport().accept_solution_state(&mut solution_ctx);
 
         let route_ctx = solution_ctx.routes.first().unwrap();
-        assert_eq!(route_ctx.route.tour.get(1).unwrap().schedule, Schedule { arrival: 20., departure: 25. });
+        assert_eq!(route_ctx.route.tour.get(1).unwrap().schedule, Schedule { arrival: 10., departure: 25. });
         assert_eq!(route_ctx.route.tour.get(2).unwrap().schedule, Schedule { arrival: 35., departure: 60. });
     }
 
@@ -228,12 +228,12 @@ mod timing {
         assert_eq!(result, Some(RouteConstraintViolation { code: 1 }));
     }
 
-    parameterized_test! {can_get_new_departure_time, (latest, optimize_whole_tour, tws, expected), {
+    parameterized_test! {can_advance_departure_time, (latest, optimize_whole_tour, tws, expected), {
         let tws = tws.into_iter().map(|(start, end)| TimeWindow::new(start, end)).collect::<Vec<_>>();
-        can_get_new_departure_time_impl(latest, optimize_whole_tour, tws, expected);
+        can_advance_departure_time_impl(latest, optimize_whole_tour, tws, expected);
     }}
 
-    can_get_new_departure_time! {
+    can_advance_departure_time! {
         case01: (None, true, vec![(0., 100.), (25., 100.), (0., 100.)], Some(5.)),
         case02: (Some(3.), true, vec![(0., 100.), (25., 100.), (0., 100.)], Some(3.)),
         case03: (Some(7.), true, vec![(0., 100.), (25., 100.), (0., 100.)], Some(5.)),
@@ -244,7 +244,7 @@ mod timing {
         case07: (None, false, vec![(0., 100.), (25., 100.), (0., 100.)], None),
     }
 
-    fn can_get_new_departure_time_impl(
+    fn can_advance_departure_time_impl(
         latest: Option<f64>,
         optimize_whole_tour: bool,
         tws: Vec<TimeWindow>,
@@ -272,13 +272,65 @@ mod timing {
             );
 
             let departure_time =
-                try_get_new_departure_time(&TestTransportCost::default(), &route_ctx, optimize_whole_tour);
+                try_advance_departure_time(&route_ctx, &TestTransportCost::default(), optimize_whole_tour);
 
             assert_eq!(departure_time, expected);
         } else {
             unreachable!()
         }
     }
+
+    parameterized_test! {can_recede_departure_time, (earliest, start_departure, latest_first_arrival, tw, duration_limit, expected), {
+        can_recede_departure_time_impl(earliest, start_departure, latest_first_arrival, TimeWindow::new(tw.0, tw.1), duration_limit, expected);
+    }}
+
+    can_recede_departure_time! {
+        case01: (Some(0.), 0., 10., (10., 20.), None, None),
+        case02: (Some(0.), 5., 10., (10., 20.), None, None),
+        case03: (Some(5.), 10., 15., (10., 20.), None, Some(5.)),
+        case04: (Some(5.), 10., 20., (10., 20.), None, Some(5.)),
+        case05: (None, 10., 50., (10., 20.), None, Some(0.)),
+        case06: (Some(5.), 10., 11., (10., 20.), None, Some(9.)),
+
+        case07: (Some(0.), 10., 20., (10., 20.), Some((20., 30.)), Some(0.)),
+        case08: (Some(0.), 10., 20., (10., 20.), Some((20., 25.)), Some(5.)),
+        case09: (Some(0.), 10., 20., (10., 20.), Some((20., 20.)), None),
+    }
+
+    fn can_recede_departure_time_impl(
+        earliest: Option<f64>,
+        start_departure: f64,
+        latest_first_arrival: f64,
+        tw: TimeWindow,
+        total_duration_limit: Option<(f64, f64)>,
+        expected: Option<f64>,
+    ) {
+        let fleet = FleetBuilder::default()
+            .add_driver(test_driver())
+            .add_vehicle(Vehicle {
+                details: vec![VehicleDetail {
+                    start: Some(VehiclePlace { location: 0, time: TimeInterval { earliest, latest: None } }),
+                    ..test_vehicle_detail()
+                }],
+                ..test_vehicle_with_id("v1")
+            })
+            .build();
+        let mut route_ctx =
+            create_route_context_with_activities(&fleet, "v1", vec![test_activity_with_location_and_tw(10, tw)]);
+        let (route, state) = route_ctx.as_mut();
+        route.tour.get_mut(0).unwrap().schedule.departure = start_departure;
+        let first = route.tour.get(1).unwrap();
+        state.put_activity_state::<f64>(LATEST_ARRIVAL_KEY, first, latest_first_arrival);
+
+        if let Some((total, limit)) = total_duration_limit {
+            state.put_route_state::<f64>(TOTAL_DURATION_KEY, total);
+            state.put_route_state::<f64>(LIMIT_DURATION_KEY, limit);
+        }
+
+        let departure_time = try_recede_departure_time(&route_ctx);
+
+        assert_eq!(departure_time, expected);
+    }
 }
 
 mod traveling {
